@@ -1,31 +1,27 @@
 """Tests for gtimelog.timelog"""
 
-import codecs
 import datetime
 import doctest
 import os
 import re
 import shutil
+import sys
 import tempfile
 import textwrap
 import time
 import unittest
-import sys
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from io import StringIO
+from io import StringIO
+from unittest import mock
 
 import freezegun
-try:
-    # Python 3
-    from unittest import mock
-except ImportError:
-    # Python 2
-    import mock
 
 from gtimelog.timelog import (
-    TimeLog, Reports, ReportRecord, Exports, TaskList, TimeCollection,
+    Exports,
+    ReportRecord,
+    Reports,
+    TaskList,
+    TimeCollection,
+    TimeLog,
 )
 
 
@@ -1007,12 +1003,12 @@ class Mixins(object):
 
     def write_file(self, filename, content):
         filename = os.path.join(self.mkdtemp(), filename)
-        with codecs.open(filename, 'w', encoding='UTF-8') as f:
+        with open(filename, 'w', encoding='utf-8') as f:
             f.write(content)
         return filename
 
 
-class TestTimeCollection(unittest.TestCase):
+class TestTimeCollection(Mixins, unittest.TestCase):
 
     def test_split_category(self):
         sp = TimeCollection.split_category
@@ -1027,6 +1023,41 @@ class TestTimeCollection(unittest.TestCase):
         self.assertEqual(sp('project: '), ('project', ''))
         self.assertEqual(sp('project:'), ('project', ''))
 
+    def test_sorted_grouped_time_collection(self):
+        # the unsorted list is nromally a TimeCollection but we fake it
+        unsorted_list = (  # list of (start-time, name, duration)
+            (10_000, 'BBB: b', 20),
+            (30_000, 'CCC: c', 10),
+            (20_000, 'Alone', 12),
+            (40_000, 'AAA: a', 15),
+        )
+        taskfile = self.write_file('tasks.txt', textwrap.dedent('''\
+            Alone
+            # comments are skipped
+            BBB: b
+            AAA: a
+            CCC: c
+        '''))
+        tasklist = TaskList(taskfile)
+        sorted_by = {
+            'start-time': ('BBB: b', 'Alone', 'CCC: c', 'AAA: a'),
+            'name': ('AAA: a', 'Alone', 'BBB: b', 'CCC: c'),
+            'duration': ('CCC: c', 'Alone', 'AAA: a', 'BBB: b'),
+            'task-list': ('Alone', 'BBB: b', 'AAA: a', 'CCC: c'),
+        }
+
+        def tc_sorted(method):
+            tc_key = TimeCollection._get_grouped_order_key
+            return tuple(name for start_time, name, duration in
+                         sorted(unsorted_list, key=tc_key(method, tasklist)))
+
+        # we could have a loop but then it wouldn't be clear with which
+        # method the assert fails
+        self.assertEqual(tc_sorted('start-time'), sorted_by['start-time'])
+        self.assertEqual(tc_sorted('name'), sorted_by['name'])
+        self.assertEqual(tc_sorted('duration'), sorted_by['duration'])
+        self.assertEqual(tc_sorted('task-list'), sorted_by['task-list'])
+
 
 class TestTaskList(Mixins, unittest.TestCase):
 
@@ -1035,27 +1066,30 @@ class TestTaskList(Mixins, unittest.TestCase):
         self.assertFalse(tasklist.check_reload())
         tasklist.reload()  # no crash
 
-    def test_parsing(self):
+    def test_parsing_and_ordering(self):
         taskfile = self.write_file('tasks.txt', textwrap.dedent('''\
             # comments are skipped
             some task
             other task
             project: do it
-            project: fix bugs
+            project:fix bugs
             misc: paperwork
         '''))
         tasklist = TaskList(taskfile)
         self.assertEqual(tasklist.groups, [
-            ('Other', ['some task', 'other task']),
-            ('misc', ['paperwork']),
             ('project', ['do it', 'fix bugs']),
+            ('misc', ['paperwork']),
+            ('Other', ['some task', 'other task']),
         ])
+        # also test that the order function works as foreseen
+        self.assertEqual(tasklist.order('project: fix bugs'), 4)
+        self.assertEqual(tasklist.order('unknown task'),  sys.maxsize)
 
     def test_unicode(self):
-        taskfile = self.write_file('tasks.txt', u'\N{SNOWMAN}')
+        taskfile = self.write_file('tasks.txt', '\N{SNOWMAN}')
         tasklist = TaskList(taskfile)
         self.assertEqual(tasklist.groups, [
-            ('Other', [u'\N{SNOWMAN}']),
+            ('Other', ['\N{SNOWMAN}']),
         ])
 
     def test_reloading(self):
@@ -1154,6 +1188,79 @@ class TestTimeLog(Mixins, unittest.TestCase):
         timelog.append('now')
         self.assertEqual(timelog.items[-1][0], datetime.datetime(2015, 5, 12, 16, 27))
 
+    @freezegun.freeze_time("2018-12-09 16:27")
+    def test_remove_last_entry(self):
+        TEST_TIMELOG = textwrap.dedent("""
+            2018-12-09 08:30: start at home
+            2018-12-09 08:40: emails
+            # comment
+            2018-12-09 12:15: coding
+        """)
+        filename = self.tempfile()
+        self.write_file(filename, TEST_TIMELOG)
+        timelog = TimeLog(filename, datetime.time(2, 0))
+        last_entry = timelog.remove_last_entry()
+        self.assertEqual(last_entry, 'coding')
+        items_after_call = [
+            (datetime.datetime(2018, 12, 9, 8, 30), 'start at home'),
+            (datetime.datetime(2018, 12, 9, 8, 40), 'emails')]
+        self.assertEqual(timelog.items, items_after_call)
+        self.assertEqual(timelog.window.items, items_after_call)
+        with open(filename) as f:
+            self.assertEqual(f.read(), textwrap.dedent("""
+                2018-12-09 08:30: start at home
+                2018-12-09 08:40: emails
+                # comment
+                ##2018-12-09 12:15: coding
+            """))
+
+        last_entry = timelog.remove_last_entry()
+        self.assertEqual(last_entry, 'emails')
+        items_after_call = [
+            (datetime.datetime(2018, 12, 9, 8, 30), 'start at home')]
+        self.assertEqual(timelog.items, items_after_call)
+        self.assertEqual(timelog.window.items, items_after_call)
+        with open(filename) as f:
+            self.assertEqual(f.read(), textwrap.dedent("""
+                2018-12-09 08:30: start at home
+                ##2018-12-09 08:40: emails
+                # comment
+                ##2018-12-09 12:15: coding
+            """))
+
+    @freezegun.freeze_time("2018-12-10 10:40")
+    def test_remove_last_entry_start_of_day(self):
+
+        TEST_TIMELOG = textwrap.dedent("""
+            2018-12-09 08:30: start at home
+            2018-12-09 08:40: emails
+
+            2018-12-10 08:30: start at home
+        """)
+
+        filename = self.tempfile()
+        self.write_file(filename, TEST_TIMELOG)
+        timelog = TimeLog(filename, datetime.time(2, 0))
+        timelog.reread()
+        last_entry = timelog.remove_last_entry()
+        self.assertEqual(last_entry, 'start at home')
+        items_after_call = [
+            (datetime.datetime(2018, 12, 9, 8, 30), 'start at home'),
+            (datetime.datetime(2018, 12, 9, 8, 40), 'emails')]
+        self.assertEqual(timelog.items, items_after_call)
+        self.assertEqual(timelog.window.items, [])
+        with open(filename) as f:
+            self.assertEqual(f.read(), textwrap.dedent("""
+                2018-12-09 08:30: start at home
+                2018-12-09 08:40: emails
+
+                ##2018-12-10 08:30: start at home
+            """))
+
+        # no further remove possible at beginning of the day:
+        last_entry = timelog.remove_last_entry()
+        self.assertIsNone(last_entry)
+
     @freezegun.freeze_time("2015-05-12 16:27")
     def test_valid_time_accepts_any_time_in_the_past_when_log_is_empty(self):
         timelog = TimeLog(StringIO(), datetime.time(2, 0))
@@ -1226,24 +1333,51 @@ class TestTimeLog(Mixins, unittest.TestCase):
                          ("15:20 did stuff", None))
 
     @freezegun.freeze_time("2015-05-12 16:27")
-    def test_parse_correction_recognizes_relative_times(self):
+    def test_parse_correction_recognizes_negative_relative_times(self):
         timelog = TimeLog(StringIO(), datetime.time(2, 0))
         self.assertEqual(timelog.parse_correction("-20 did stuff"),
                          ("did stuff", datetime.datetime(2015, 5, 12, 16, 7)))
 
     @freezegun.freeze_time("2015-05-12 16:27")
-    def test_parse_correction_ignores_relative_times_before_last_entry(self):
+    def test_parse_correction_recognizes_positive_relative_times(self):
+        timelog = TimeLog(StringIO("2015-05-12 15:50: stuff"),
+                          datetime.time(2, 0))
+        self.assertEqual(timelog.parse_correction("+20 did stuff"),
+                         ("did stuff", datetime.datetime(2015, 5, 12, 16, 10)))
+
+    @freezegun.freeze_time("2015-05-12 16:27")
+    def test_parse_correction_ignores_positive_relative_times_without_initial_entry(self):
+        timelog = TimeLog(StringIO(), datetime.time(2, 0))
+        self.assertEqual(timelog.parse_correction("+20 did stuff"),
+                         ("+20 did stuff", None))
+
+    @freezegun.freeze_time("2015-05-12 16:27")
+    def test_parse_correction_ignores_negative_relative_times_before_last_entry(self):
         timelog = TimeLog(StringIO("2015-05-12 16:00: stuff"),
                           datetime.time(2, 0))
         self.assertEqual(timelog.parse_correction("-30 did stuff"),
                          ("-30 did stuff", None))
 
     @freezegun.freeze_time("2015-05-12 16:27")
-    def test_parse_correction_ignores_bad_relative_times(self):
+    def test_parse_correction_ignores_positive_relative_times_in_the_future(self):
+        timelog = TimeLog(StringIO("2015-05-12 15:50: stuff"),
+                          datetime.time(2, 0))
+        self.assertEqual(timelog.parse_correction("+40 did stuff"),
+                         ("+40 did stuff", None))
+
+    @freezegun.freeze_time("2015-05-12 16:27")
+    def test_parse_correction_ignores_bad_negative_relative_times(self):
         timelog = TimeLog(StringIO(), datetime.time(2, 0))
         self.assertEqual(timelog.parse_correction("-200 did stuff"),
                          ("-200 did stuff", None))
 
+    @freezegun.freeze_time("2015-05-12 16:27")
+    def test_parse_correction_ignores_bad_positive_relative_times(self):
+        timelog = TimeLog(StringIO("2015-05-12 15:50: stuff"),
+                          datetime.time(2, 0))
+        self.assertEqual(timelog.parse_correction("+200 did stuff"),
+                         ("+200 did stuff", None))
+
 
 class TestTotals(unittest.TestCase):
 
