@@ -2,24 +2,18 @@
 Non-GUI bits of gtimelog.
 """
 
-from __future__ import unicode_literals
-
-import codecs
 import collections
 import csv
 import datetime
 import os
+import re
 import socket
 import sys
-import re
 from collections import defaultdict
 from hashlib import md5
 from operator import itemgetter
 
 
-PY3 = sys.version_info[0] >= 3
-
-
 def as_minutes(duration):
     """Convert a datetime.timedelta to an integer number of minutes."""
     return duration.days * 24 * 60 + duration.seconds // 60
@@ -118,10 +112,10 @@ def next_month(date):
         return datetime.date(date.year, date.month + 1, 1)
 
 
-def uniq(l):
+def uniq(items):
     """Return list with consecutive duplicates removed."""
-    result = l[:1]
-    for item in l[1:]:
+    result = items[:1]
+    for item in items[1:]:
         if item != result[-1]:
             result.append(item)
     return result
@@ -249,9 +243,10 @@ class TimeCollection(object):
         Return a tuple (category, task).
         """
         if ': ' in entry:
-            return tuple(entry.split(': ', 1))
+            cat, tsk = entry.split(': ', 1)
+            return cat.strip(), tsk.strip()
         elif entry.endswith(':'):
-            return entry.partition(':')[0], ''
+            return entry.partition(':')[0].strip(), ''
         else:
             return None, entry
 
@@ -273,7 +268,8 @@ class TimeCollection(object):
                 count += 1
         return count
 
-    def grouped_entries(self, skip_first=True):
+    def grouped_entries(self, skip_first=True,
+                        sorted_by='start-time', sorted_tasks=None):
         """Return consolidated entries (grouped by entry title).
 
         Returns two lists: work entries and slacking entries.  Slacking
@@ -299,8 +295,9 @@ class TimeCollection(object):
                 start = min(start, old_start)
                 duration += old_duration
             entries[entry] = (start, entry, duration)
-        work = sorted(work.values())
-        slack = sorted(slack.values())
+        key_func = self._get_grouped_order_key(sorted_by, sorted_tasks)
+        work = sorted(work.values(), key=key_func)
+        slack = sorted(slack.values(), key=key_func)
         return work, slack
 
     def categorized_work_entries(self, skip_first=True):
@@ -367,6 +364,26 @@ class TimeCollection(object):
                 total_work += duration
         return total_work, total_slacking
 
+    @classmethod
+    def _get_grouped_order_key(cls, sorted_by, sorted_tasks):
+        """
+        Returns a callable usable as the `key` argument of sorted().
+
+        The parameter 'x' to be sorted is deemed a list item as returned by
+        TimeCollection.grouped_entries
+        """
+        # name is deemed unique as this function is used for grouped entries,
+        # hence sufficient to fully sort a list
+        if sorted_by == 'start-time':
+            return None  # hence sort by x
+        elif sorted_by == 'name':
+            return lambda x: x[1]
+        elif sorted_by == 'duration':  # return (duration, start-time, name)
+            return lambda x: (x[2], x[0], x[1])
+        elif sorted_by == 'task-list':
+            # name is also sent to order unknown entries in a stable way
+            return lambda x: (sorted_tasks.order(x[1]), x[1])
+
 
 class TimeWindow(TimeCollection):
     """A window into a time log.
@@ -422,7 +439,7 @@ class Exports(object):
 
         The file has two columns: task title and time (in minutes).
         """
-        writer = CSVWriter(output)
+        writer = csv.writer(output)
         if title_row:
             writer.writerow(["task", "time (minutes)"])
         work, slack = self.window.grouped_entries()
@@ -438,13 +455,13 @@ class Exports(object):
         The file has four columns: date, time from midnight til arrival at
         work, slacking, and work (in decimal hours).
         """
-        writer = CSVWriter(output)
+        writer = csv.writer(output)
         if title_row:
             writer.writerow(["date", "day-start (hours)",
                              "slacking (hours)", "work (hours)"])
 
         # sum timedeltas per date
-        # timelog must be cronological for this to be dependable
+        # timelog must be chronological for this to be dependable
 
         d0 = datetime.timedelta(0)
         days = {} # date -> [time_started, slacking, work]
@@ -553,11 +570,11 @@ class Reports(object):
                         continue # skip empty "arrival" entries
 
                     entry = entry[:1].upper() + entry[1:]
-                    output.write(u"  %-61s  %+5s\n" %
+                    output.write("  %-61s  %+5s\n" %
                                  (entry, format_duration_short(duration)))
 
                 output.write('-' * 70 + '\n')
-                output.write(u"%+70s\n" % format_duration_short(totals[cat]))
+                output.write("%+70s\n" % format_duration_short(totals[cat]))
                 output.write('\n')
         output.write("Total work done this %s: %s\n" %
                      (period_name, format_duration_short(total_work)))
@@ -632,7 +649,7 @@ class Reports(object):
         if no_cat is not None:
             items.append(('(none)', no_cat))
         for cat, duration in items:
-            output.write(u"%-62s  %s\n" % (
+            output.write("%-62s  %s\n" % (
                 cat, format_duration_long(duration)))
         output.write('\n')
 
@@ -669,7 +686,7 @@ class Reports(object):
                     cat, datetime.timedelta(0)) + duration
 
                 entry = entry[:1].upper() + entry[1:]
-                output.write(u"%-62s  %s\n" %
+                output.write("%-62s  %s\n" %
                              (entry, format_duration_long(duration)))
             output.write('\n')
         output.write("Total work done this %s: %s\n" %
@@ -684,7 +701,7 @@ class Reports(object):
 
     def weekly_report_subject(self, who):
         week = self.window.min_timestamp.isocalendar()[1]
-        return u'Weekly report for %s (week %02d)' % (who, week)
+        return 'Weekly report for %s (week %02d)' % (who, week)
 
     def weekly_report(self, output, email, who):
         if self.style == 'categorized':
@@ -706,7 +723,7 @@ class Reports(object):
 
     def monthly_report_subject(self, who):
         month = self.window.min_timestamp.strftime('%Y/%m')
-        return u'Monthly report for %s (%s)' % (who, month)
+        return 'Monthly report for %s (%s)' % (who, month)
 
     def monthly_report(self, output, email, who):
         if self.style == 'categorized':
@@ -730,7 +747,7 @@ class Reports(object):
         min = self.window.min_timestamp.strftime('%Y-%m-%d')
         max = self.window.max_timestamp - datetime.timedelta(1)
         max = max.strftime('%Y-%m-%d')
-        return u'Custom date range report for %s (%s - %s)' % (who, min, max)
+        return 'Custom date range report for %s (%s - %s)' % (who, min, max)
 
     def custom_range_report_categorized(self, output, email, who):
         """Format a custom range report with entries displayed under categories."""
@@ -744,8 +761,8 @@ class Reports(object):
         weekday_names = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
         weekday = weekday_names[self.window.min_timestamp.weekday()]
         week = self.window.min_timestamp.isocalendar()[1]
-        return (u"{0:%Y-%m-%d} report for {who}"
-                u" ({weekday}, week {week:0>2})".format(
+        return ("{0:%Y-%m-%d} report for {who}"
+                " ({weekday}, week {week:0>2})".format(
                     self.window.min_timestamp, who=who,
                     weekday=weekday, week=week))
 
@@ -756,8 +773,8 @@ class Reports(object):
         """
         window = self.window
         if self.email_headers:
-            output.write(u"To: %s\n" % email)
-            output.write(u"Subject: %s\n" % self.daily_report_subject(who))
+            output.write("To: %s\n" % email)
+            output.write("Subject: %s\n" % self.daily_report_subject(who))
             output.write('\n')
         items = list(window.all_entries())
         if not items:
@@ -773,7 +790,7 @@ class Reports(object):
         if work:
             for start, entry, duration in work:
                 entry = entry[:1].upper() + entry[1:]
-                output.write(u"%-62s  %s\n" % (entry,
+                output.write("%-62s  %s\n" % (entry,
                                                format_duration_long(duration)))
                 cat, task = TimeCollection.split_category(entry)
                 categories[cat] = categories.get(
@@ -790,7 +807,7 @@ class Reports(object):
         if slack:
             for start, entry, duration in slack:
                 entry = entry[:1].upper() + entry[1:]
-                output.write(u"%-62s  %s\n" % (entry,
+                output.write("%-62s  %s\n" % (entry,
                                                format_duration_long(duration)))
             output.write('\n')
         output.write("Time spent slacking: %s\n" %
@@ -922,9 +939,8 @@ class TimeLog(TimeCollection):
                 self.filename.seek(0)
                 self.items = self._read(self.filename)
             else:
-                with open(self.filename, 'rb') as f:
-                    data = f.read()
-                self.items = self._read(data.decode('UTF-8').splitlines())
+                with open(self.filename, encoding='utf-8') as f:
+                    self.items = self._read(f)
         except IOError:
             self.items = []
         self.window = self.window_for_day(self.day)
@@ -993,13 +1009,39 @@ class TimeLog(TimeCollection):
         max = max + datetime.timedelta(1)
         return self.window_for(min, max)
 
+    def remove_last_entry(self):
+        self.check_reload()
+        if not self.window.items:
+            # last day's entries list is empty, so nothing to remove
+            return None
+        with open(self.filename, "r", encoding='utf-8') as f:
+            lines = f.readlines()
+        for idx, line in enumerate(reversed(lines), start=1):
+            time, sep, entry = line.partition(': ')
+            if not sep:
+                continue
+            try:
+                time = parse_datetime(time)
+            except ValueError:
+                continue
+            last_entry = entry.strip()
+            break
+        else:
+            # maybe timelog.txt got replaced after we did check_reload() but
+            # before we re-read it?
+            return None  # pragma: nocover
+        lines[-idx] = '##' + lines[-idx]
+        with open(self.filename, "w", encoding='utf-8') as f:
+            f.writelines(lines)
+        self.reread()
+        return last_entry
+
     def raw_append(self, line, need_space):
         """Append a line to the time log file."""
-        f = codecs.open(self.filename, "a", encoding='UTF-8')
-        if need_space:
-            f.write('\n')
-        f.write(line + '\n')
-        f.close()
+        with open(self.filename, "a", encoding='utf-8') as f:
+            if need_space:
+                f.write('\n')
+            f.write(line + '\n')
         self.last_mtime = get_mtime(self.filename)
 
     def append(self, entry, now=None):
@@ -1040,7 +1082,7 @@ class TimeLog(TimeCollection):
         """
         now = None
         date_match = re.match(r'(\d\d):(\d\d)\s+', entry)
-        delta_match = re.match(r'-([1-9]\d?|1\d\d)\s+', entry)
+        delta_match = re.match(r'[\-+]([1-9]\d?|1\d\d)\s+', entry)
         if date_match:
             h = int(date_match.group(1))
             m = int(date_match.group(2))
@@ -1055,12 +1097,19 @@ class TimeLog(TimeCollection):
                     now = None
         if delta_match:
             seconds = int(delta_match.group()) * 60
-            now = datetime.datetime.now().replace(second=0, microsecond=0)
-            now += datetime.timedelta(seconds=seconds)
-            if self.valid_time(now):
-                entry = entry[delta_match.end():]
+            # If positive, offset from the end of the last entry.
+            if seconds >= 0:
+                now = self.window.last_time()
+            # Otherwise, offset from the current time.
             else:
-                now = None
+                now = datetime.datetime.now().replace(second=0, microsecond=0)
+
+            if now is not None:
+                now += datetime.timedelta(seconds=seconds)
+                if self.valid_time(now):
+                    entry = entry[delta_match.end():]
+                else:
+                    now = None
         return entry, now
 
 
@@ -1080,6 +1129,10 @@ class TaskList(object):
 
     A TaskList has an attribute 'groups' which is a list of tuples
     (group_name, list_of_group_items).
+
+    It also has an attribute 'task_order' which is a dictionary of task names,
+    potentially prefixed by 'group_name: ', with their value being their
+    original order in the task list.
     """
 
     other_title = 'Other'
@@ -1107,40 +1160,39 @@ class TaskList(object):
     def load(self):
         """Load task list from a file named self.filename."""
         groups = {}
+        task_order = {}
+        others = []
         self.last_mtime = get_mtime(self.filename)
         try:
-            with codecs.open(self.filename, encoding='UTF-8') as f:
-                for line in f:
+            with open(self.filename, encoding='utf-8') as f:
+                for index, line in enumerate(f):
                     line = line.strip()
                     if not line or line.startswith('#'):
                         continue
-                    if ':' in line:
+                    if ':' in line:  # tasks with group prefix
                         group, task = [s.strip() for s in line.split(':', 1)]
-                    else:
-                        group, task = self.other_title, line
-                    groups.setdefault(group, []).append(task)
+                        groups.setdefault(group, []).append(task)
+                        task_order[group + ': ' + task] = index
+                    else:  # "other" tasks
+                        others.append(line)
+                        task_order[line] = index
         except IOError:
             pass # the file's not there, so what?
-        self.groups = sorted(groups.items())
+        # append the "other" tasks at the end
+        self.groups = list(groups.items())
+        if others:
+            self.groups.append((self.other_title, others))
+        self.task_order = task_order
 
     def reload(self):
         """Reload the task list."""
         self.load()
 
+    def order(self, value):
+        """
+        Return the order index of a value in the task order list
 
-class CSVWriter(object):
-
-    def __init__(self, *args, **kw):
-        self._writer = csv.writer(*args, **kw)
-
-    if PY3:  # pragma: PY3
-        def writerow(self, row):
-            self._writer.writerow(row)
-    else:  # pragma: PY2
-        def writerow(self, row):
-            self._writer.writerow([s.encode('UTF-8') if isinstance(s, unicode)
-                                   else s for s in row])
-
-    def writerows(self, rows):
-        for row in rows:
-            self.writerow(row)
+        If the value isn't in the task order dictionary, it returns a value
+        bigger than any index.
+        """
+        return self.task_order.get(value, sys.maxsize)
